#![no_std]
#![no_main]
#![feature(abi_x86_interrupt)]

use multiboot2::BootInformationHeader;
use x86_64::registers::control::Cr3;

use crate::mem::FrameAllocator;

#[macro_use]
mod logger;
mod interrupts;
mod gdt;
mod utils;
mod events;
mod mem;




#[no_mangle]
extern "C" fn kmain(mb2_info_addr: usize) -> ! {
    logger::init(log::LevelFilter::Trace).unwrap();
    gdt::init();
    interrupts::init_idt();
    let mem_info = mem::MemInfo::load(mb2_info_addr);
    let mut alloc = mem::area_frame_alloc::AreaFrameAllocator::new(mem_info);
    
    for i in 0.. {
    if let None = alloc.allocate_frame() {
        println!("allocated {} frames", i);
        break;
    }
}
    
    // let (level_4_page_table, _) = Cr3::read();
    // println!("Level 4 page table at: {:?}", level_4_page_table.start_address());


    utils::hcf();
}


#[panic_handler]
fn panic(pi: &core::panic::PanicInfo) -> ! {
    log::error!("{pi}");
    utils::hcf();
}


